/*****************************************************************************/
/*                     WiNRADiO G65DDC EXCALIBUR SIGMA                       */
/*                                SDK DEMO                                   */
/*             Receiver selection and "No receiver found" dialogs            */
/*                        (c) Copyright RADIXON 2020                         */
/*****************************************************************************/

#ifndef __U_RECEIVERSELECTOR_H__
#define __U_RECEIVERSELECTOR_H__

#include <wx/wx.h>
#include <wx/listctrl.h>
#include "u_receiver.h"

//TReceiverSelector window appears when more than one receivers are detected
//the user can select one of them
class TReceiverSelector:public wxDialog
{
public:
    static TReceiver* SelectReceiver(void);

    inline int GetSelectedReceiverIndex(void) const
    {
        return m_SelectedReceiverIndex;
    }

    TReceiverSelector(IG65DDCDeviceEnumerator *Enumerator);
    ~TReceiverSelector(void);
private:
    wxRadioBox      *m_RadioBox;
    wxButton        *m_BOK;
    wxButton        *m_BCancel;
    int             m_SelectedReceiverIndex;

    void OnOk(wxCommandEvent &Event);

    DECLARE_EVENT_TABLE()
};

//TNoReceiverFoundWindow window appears when no receiver is detected
class TNoReceiverFoundWindow:public wxDialog
{
public:
    TNoReceiverFoundWindow(void);
    ~TNoReceiverFoundWindow(void);
private:
    wxButton    *m_BTryAgain;
    wxButton    *m_BDemo;
    wxButton    *m_BCancel;
    wxButton    *m_BNetwork;

    void OnTryAgain(wxCommandEvent &Event);
    void OnDemo(wxCommandEvent &Event);
    void OnScanNetwork(wxCommandEvent &Event);

    DECLARE_EVENT_TABLE()
};

class TScanNetwork:public wxDialog,public IG65DDCNetworkEnumeratorCallback
{
public:
    virtual void G65DDC_CALLBACK G65DDC_DeviceFound(uint32_t IpAddress,uint16_t Port,const char *SerialNumber,uint32_t Flags);

    inline wxString GetAddress(void) const
    {
        return m_Address;
    }

    TScanNetwork(void);
    ~TScanNetwork(void);
private:
    class TDeviceInfo
    {
    public:
        inline TDeviceInfo(const char *SerialNumber,uint32_t IpAddress,uint16_t Port)
        {
            m_SerialNumber=SerialNumber;
            m_IpAddress=IpAddress;
            m_Port=Port;
            m_Countdown=1;
        }

        wxString                m_SerialNumber;
        uint32_t                m_IpAddress;
        uint16_t                m_Port;
        int                     m_Countdown;
    };

    IG65DDCNetworkEnumerator    *m_Enumerator;
    wxTimer                     m_Timer;
    wxVector<TDeviceInfo*>      m_FoundDevices;
    wxMutex                     m_Mutex;
    wxListView                  *m_LV;
    wxButton                    *m_BOK;
    wxButton                    *m_BCancel;
    wxString                    m_Address;

    void OnTimer(wxTimerEvent& Event);
    void OnDeviceListChanged(wxCommandEvent &Event);
    void OnOk(wxCommandEvent &Event);
    void OnCancel(wxCommandEvent &Event);

    void OnDeselected(wxListEvent& Event);
    void OnSelected(wxListEvent& Event);

    DECLARE_EVENT_TABLE()
};


#endif
