/*****************************************************************************/
/*                      WiNRADiO G65DDC EXCALIBUR SIGMA                      */
/*                       Coherent mode SDK DEMO - ETC                        */
/*                             Main GUI window                               */
/*                        (c) Copyright RADIXON 2023                         */
/*****************************************************************************/

#include "u_mainwindow.h"
#include "u_fft.h"

enum
{
    ID_POWER=wxID_HIGHEST,
    ID_1PPS,
    ID_DDC1,
    ID_DDC1_BW,
    ID_TIMER,
    ID_DDC1_FREQUENCY,
    ID_ATTEN,
    ID_PREAMP,

    ID_RX_BUTTON,

    ID_PHASE_SHIFT=ID_RX_BUTTON+128
};

BEGIN_EVENT_TABLE(main_window, wxFrame)
    EVT_TOGGLEBUTTON(ID_POWER,main_window::on_click_power)
    EVT_TOGGLEBUTTON(ID_1PPS,main_window::on_click_1pps)
    EVT_TOGGLEBUTTON(ID_DDC1,main_window::on_click_ddc1)
    EVT_TOGGLEBUTTON(ID_PREAMP,main_window::on_click_preamp)
    EVT_TIMER(ID_TIMER,main_window::on_timer)
    EVT_COMBOBOX(ID_DDC1_BW,main_window::on_ddc1_bw_chaned)
    EVT_COMBOBOX(ID_ATTEN,main_window::on_atten_chaned)
    EVT_NUMERICEDIT(ID_DDC1_FREQUENCY,main_window::on_ddc1_frequency_changed)
END_EVENT_TABLE()



wxString format_frequency(int64_t frequency)
{
 char s[64];
 const char *units;
 double f;
 int i;

    f=frequency;

    if(fabs(f)<1000)
    {
        sprintf(s,"%.0f Hz",f);
    }
    else
    {
        f=frequency;

        f/=1000;

        if(fabs(f)<1000)
        {
            i=sprintf(s,"%.3f",f);
            units="kHz";
        }
        else
        {
            f/=1000;

            if(fabs(f)<1000)
            {
                i=sprintf(s,"%.6f",f);
                units="MHz";
            }
            else
            {
                i=sprintf(s,"%.9f",f/1000);
                units="GHz";
            }
        }

        while(--i>=0)
        {
            if(s[i]=='.')
            {
                s[i]='\0';
                break;
            }

            if(s[i]!='0')
            {
                i++;
                break;
            }
        }

        s[i]=' ';
        strcpy(s+i+1,units);
    }

    return wxString(s,wxConvLocal);
}

main_window::main_window(const std::vector<rx_client*> &clients):wxFrame(NULL,wxID_ANY,APP_TITLE wxT(" - Client"),wxDefaultPosition,wxSize(1000,900))
{
 wxRect rect1,rect2;
 wxSizer *main_sizer,*sizer,*phase_sizer,*status_sizer;
 wxString s;
 wxToggleButton *b;
 TPhase **phases;
 wxStaticText *st;
 TFloatEdit *fe;
 wxSize size;

    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));

    m_clients=clients;

    for(auto client:m_clients)
    {
        client->set_callback(this);
    }

    m_connected=false;
    m_power=false;
    m_active_rx_index=0;

    load_settings();

    main_sizer=new wxBoxSizer(wxVERTICAL);

    m_ddc1_frequency=15000000;


    sizer=new wxBoxSizer(wxHORIZONTAL);
    main_sizer->Add(sizer,0,wxEXPAND|wxTOP|wxBOTTOM,4);

    m_eddc1_frequency=new TFrequencyEdit(this,ID_DDC1_FREQUENCY,m_ddc1_frequency,0,250000000);
    size=m_eddc1_frequency->GetTextExtent(wxT("250.000000 MHz"));
    size=m_eddc1_frequency->GetSizeFromTextSize(size.x+20);
    m_eddc1_frequency->SetInitialSize(size);

    sizer->Add(m_eddc1_frequency,0,wxALIGN_CENTER_VERTICAL|wxALIGN_LEFT);

    m_bddc1=new wxToggleButton(this,ID_DDC1,wxT("DDC1"),wxDefaultPosition,wxDefaultSize,wxBU_EXACTFIT);
    sizer->Add(m_bddc1,0,wxALIGN_CENTER_VERTICAL|wxALIGN_LEFT|wxLEFT,4);

    m_cbddc1_bw=new wxComboBox(this,ID_DDC1_BW,wxEmptyString,wxDefaultPosition,wxDefaultSize,0,0,wxCB_DROPDOWN|wxCB_READONLY|wxTE_CENTRE);
    size=m_cbddc1_bw->GetTextExtent(wxT("64.0 MHz"));
    size=m_cbddc1_bw->GetSizeFromTextSize(size.x+20);
    m_cbddc1_bw->SetInitialSize(size);
    sizer->Add(m_cbddc1_bw,0,wxALIGN_CENTER_VERTICAL|wxALIGN_LEFT|wxLEFT,4);

    m_lddc1_rbw=new wxStaticText(this,wxID_ANY,wxEmptyString);
    sizer->Add(m_lddc1_rbw,0,wxALIGN_CENTER_VERTICAL|wxALIGN_LEFT|wxLEFT,4);

    //Very simplified version of example how to build list of available DDC1 bandwidhts
    //using hardcoded array of DDC infos. It would be better to retrieve available DDCs
    //from each server and use only such DDCs which are available on all the servers,
    //but for purposes of this demo app hardcoded list is enough. Keep in mind, if the
    //G65DDC receiver is connected to the USB2 port its max DDC bandwidth is limited.
    for(int i=0;i<DDC_TYPE_COUNT;i++)
    {
        m_cbddc1_bw->Append(wxT(" ")+format_frequency(ddc_infos[i].bandwidth));
    }

    m_cbddc1_bw->SetSelection(0);

    for(uint32_t i=0;i<m_clients.size();)
    {
        sizer=new wxBoxSizer(wxHORIZONTAL);
        main_sizer->Add(sizer,0,wxEXPAND);

        for(int j=0;j<4 && i<m_clients.size();j++,i++)
        {
            b=new wxToggleButton(this,ID_RX_BUTTON+i,m_clients[i]->get_remote_host_desc());
            sizer->Add(b,1,wxEXPAND);

            m_rx_buttons.push_back(b);
        }
    }

    Connect(ID_RX_BUTTON,ID_RX_BUTTON+m_clients.size()-1,wxEVT_COMMAND_TOGGLEBUTTON_CLICKED,wxCommandEventHandler(main_window::on_click_rx_button));

    m_rx_buttons[m_active_rx_index]->SetValue(true);

    m_ddc1_spectrum=new TSpectrum(this,wxID_ANY,wxDefaultPosition,wxSize(-1,200));
    main_sizer->Add(m_ddc1_spectrum,0,wxEXPAND);

    m_panel=new wxPanel(this,wxID_ANY,wxDefaultPosition,wxDefaultSize);
    main_sizer->Add(m_panel,1,wxEXPAND);

    sizer=new wxBoxSizer(wxVERTICAL);
    m_panel->SetSizer(sizer);

    for(uint32_t i=0;i<m_clients.size();i++)
    {
        phase_sizer=new wxBoxSizer(wxVERTICAL);
        sizer->Add(phase_sizer,1,wxEXPAND);

        m_phase_sizers.push_back(phase_sizer);

        //panel=new wxPanel(m_panel,wxID_ANY,wxDefaultPosition,wxSize(-1,25));
        status_sizer=new wxBoxSizer(wxHORIZONTAL);
        phase_sizer->Add(status_sizer,0,wxEXPAND|wxTOP|wxBOTTOM|wxLEFT,2);

        st=new wxStaticText(m_panel,wxID_ANY,wxT("Phase shift"));
        status_sizer->Add(st,0,wxALIGN_CENTER_VERTICAL);
        m_lphase_shift.push_back(st);

        fe=new TFloatEdit(m_panel,ID_PHASE_SHIFT+i,0,-18000,18000,2);
        size=fe->GetTextExtent(wxT("+180.00"));
        size=fe->GetSizeFromTextSize(size.x+20);
        fe->SetInitialSize(size);
        status_sizer->Add(fe,0,wxALIGN_CENTER_VERTICAL|wxLEFT,4);
        m_ephase_shift.push_back(fe);

        st=new wxStaticText(m_panel,wxID_ANY,wxT("State"));
        status_sizer->Add(st,0,wxALIGN_CENTER_VERTICAL|wxLEFT,8);
        m_lstate.push_back(st);

        size=GetTextExtent(wxT("TRIGGER NOT SENT"));
        st=new wxStaticText(m_panel,wxID_ANY,wxT("N/A"),wxDefaultPosition,wxSize(size.x,-1),wxST_NO_AUTORESIZE);
        status_sizer->Add(st,0,wxALIGN_CENTER_VERTICAL|wxLEFT,4);
        m_lstate_value.push_back(st);

        st=new wxStaticText(m_panel,wxID_ANY,wxT("Event counter"));
        status_sizer->Add(st,0,wxALIGN_CENTER_VERTICAL|wxLEFT,8);
        m_l1pps_event_counter.push_back(st);

        size=GetTextExtent(wxT("000,000"));
        st=new wxStaticText(m_panel,wxID_ANY,wxT("N/A"),wxDefaultPosition,wxSize(size.x,-1),wxST_NO_AUTORESIZE);
        status_sizer->Add(st,0,wxALIGN_CENTER_VERTICAL|wxLEFT,4);
        m_l1pps_event_counter_value.push_back(st);

        st=new wxStaticText(m_panel,wxID_ANY,wxT("ADC period counter"));
        status_sizer->Add(st,0,wxALIGN_CENTER_VERTICAL|wxLEFT,8);
        m_l1pps_adc_period_counter.push_back(st);

        size=GetTextExtent(wxT("000,000,000,000"));
        st=new wxStaticText(m_panel,wxID_ANY,wxT("N/A"),wxDefaultPosition,wxSize(size.x,-1),wxST_NO_AUTORESIZE);
        status_sizer->Add(st,0,wxALIGN_CENTER_VERTICAL|wxLEFT,4);
        m_l1pps_adc_period_counter_value.push_back(st);

        st=new wxStaticText(m_panel,wxID_ANY,wxT("DDC1 sample counter"));
        status_sizer->Add(st,0,wxALIGN_CENTER_VERTICAL|wxLEFT,8);
        m_l1pps_ddc1_sample_counter.push_back(st);

        size=GetTextExtent(wxT("000,000,000.0"));
        st=new wxStaticText(m_panel,wxID_ANY,wxT("N/A"),wxDefaultPosition,wxSize(size.x,-1),wxST_NO_AUTORESIZE);
        status_sizer->Add(st,0,wxALIGN_CENTER_VERTICAL|wxLEFT,4);
        m_l1pps_ddc1_sample_counter_value.push_back(st);

        st=new wxStaticText(m_panel,wxID_ANY,wxT("DDC1 ADC counter"));
        status_sizer->Add(st,0,wxALIGN_CENTER_VERTICAL|wxLEFT,8);
        m_l1pps_ddc1_adc_counter.push_back(st);

        size=GetTextExtent(wxT("000,000,000,000"));
        st=new wxStaticText(m_panel,wxID_ANY,wxT("N/A"),wxDefaultPosition,wxSize(size.x,-1),wxST_NO_AUTORESIZE);
        status_sizer->Add(st,0,wxALIGN_CENTER_VERTICAL|wxLEFT,4);
        m_l1pps_ddc1_adc_counter_value.push_back(st);

        phases=new TPhase*[m_clients.size()];

        m_phases.push_back(phases);

        for(uint32_t j=0;j<m_clients.size();j++)
        {
            if(i!=j)
            {
                phases[j]=new TPhase(m_panel,wxID_ANY);
                phase_sizer->Add(phases[j],1,wxEXPAND);
                phases[j]->SetLabel(m_clients[j]->get_remote_host_desc());
            }
            else
            {
                phases[j]=NULL;
            }
        }

        phase_sizer->ShowItems(false);
    }

    Connect(ID_PHASE_SHIFT,ID_PHASE_SHIFT+m_clients.size()-1,wxEVT_COMMAND_NUMERIC_EDIT_CHANGED,wxNotifyEventHandler(main_window::on_phase_shift_changed));

    m_phase_sizers[0]->ShowItems(true);

    sizer=new wxBoxSizer(wxHORIZONTAL);
    main_sizer->Add(sizer,0,wxEXPAND|wxTOP|wxBOTTOM,4);

    m_latten=new wxStaticText(this,wxID_ANY,wxT("Attenuator"));
    sizer->Add(m_latten,0,wxALIGN_CENTER_VERTICAL|wxALIGN_LEFT|wxLEFT,1);

    m_cbatten=new wxComboBox(this,ID_ATTEN,wxEmptyString,wxDefaultPosition,wxDefaultSize,0,0,wxCB_DROPDOWN|wxCB_READONLY);
    size=m_cbatten->GetTextExtent(wxT("21 dB"));
    size=m_cbatten->GetSizeFromTextSize(size.x+20);
    m_cbatten->SetInitialSize(size);
    sizer->Add(m_cbatten,0,wxALIGN_CENTER_VERTICAL|wxALIGN_LEFT|wxLEFT,4);

    for(int i=0;i<=7;i++)
    {
        s.sprintf(wxT("%d dB"),i*3);
        m_cbatten->Append(s);
    }

    m_cbatten->SetSelection(0);

    m_bpreamp=new wxToggleButton(this,ID_PREAMP,wxT("Preamp"));
    sizer->Add(m_bpreamp,0,wxALIGN_CENTER_VERTICAL|wxALIGN_LEFT|wxLEFT,4);

    sizer->AddStretchSpacer();

    m_b1pps=new wxToggleButton(this,ID_1PPS,wxT("1PPS"));
    sizer->Add(m_b1pps,0,wxALIGN_CENTER_VERTICAL|wxALIGN_RIGHT|wxRIGHT,8);

    m_bpower=new wxToggleButton(this,ID_POWER,wxT("Power"));
    sizer->Add(m_bpower,0,wxALIGN_CENTER_VERTICAL|wxALIGN_RIGHT);

    SetSizer(main_sizer);

    Layout();

    SetClientSize(1200,800);

    CenterOnScreen();

    m_config_window=new config_window(this);

    rect1=GetRect();
    rect2=m_config_window->GetRect();

    m_config_window->SetPosition(wxPoint(rect1.GetLeft()-rect2.GetWidth(),rect1.GetTop()+(rect1.GetWidth()-rect2.GetWidth())/2));
    m_config_window->Show();

    update_state();
    update_ddc1_bw();

    m_ddc1_spectrum->SetCenterFrequency(m_ddc1_frequency);

    m_timer.SetOwner(this,ID_TIMER);
    m_timer.Start(500);
}

main_window::~main_window(void)
{
    disconnect();

    save_settings();

    delete m_config_window;

    for(auto client:m_clients)
    {
        delete client;
    }

    for(auto phase:m_phases)
    {
        delete phase;
    }

    free_buffers();
}

void main_window::load_settings(void)
{

}

void main_window::save_settings(void)
{
}

void main_window::connect_disconnect(void)
{
    if(m_connected)
    {
        disconnect();
    }
    else
    {
        connect();
    }
}


void main_window::connect(void)
{
 uint32_t error;
 char s[512];

    for(size_t i=0;i<m_clients.size();i++)
    {
        error=m_clients[i]->connect();

        if(error!=0)
        {
            sprintf(s,"Failed to connect to %s:%u, error=%u.",
                m_clients[i]->get_address_str().c_str(),m_clients[i]->get_port(),error);

            while(i-->0)
            {
                m_clients[i]->disconnect();
            }

            wxMessageBox(s,wxT("Error"),wxICON_ERROR,this);
            return;
        }
    }

    m_connected=true;

    connection_state_changed();
}

void main_window::disconnect(void)
{
    for(auto client:m_clients)
    {
        client->disconnect();
    }

    m_connected=false;

    connection_state_changed();
}

void main_window::connection_state_changed(void)
{
    if(!m_connected)
    {
        m_power=false;
    }

    m_config_window->connection_state_changed();

    update_state();

    if(m_connected)
    {
        set_1pps();
        set_atten();
        set_preamp();

        for(size_t i=0;i<m_clients.size();i++)
        {
            m_clients[i]->set_ddc1_frequency(m_ddc1_frequency);
            m_clients[i]->set_ddc1_phase_shift((double)m_ephase_shift[i]->Get()*0.01);
        }

        trigger();
    }
}

void main_window::trigger(void)
{
    if(m_clients[0]->trigger()==0)
    {
        for(size_t i=0;i<m_clients.size();i++)
        {
            m_clients[i]->trigger_test();
        }
    }
}

void main_window::on_click_power(wxCommandEvent &event)
{
 bool power;

    power=m_bpower->GetValue();

    for(auto client:m_clients)
    {
        client->set_power(power);
    }

    m_power=power;

    update_state();
}


void main_window::on_click_ddc1(wxCommandEvent &event)
{
 uint32_t buffer_size;

    if(m_bddc1->GetValue())
    {
        GetNormalizedWindowCoeffs(m_fft_coeffs,m_fft_size);

        m_ddc1_sample_set_index.clear();

        buffer_size=2*sizeof(float)*m_ddc1_sample_sets_to_send;

        for(size_t i=0;i<m_clients.size();i++)
        {
            for(size_t j=0;j<m_clients.size();j++)
            {
                if(m_phases[i][j])
                {
                    m_phases[i][j]->Clear();
                }
            }

            m_ddc1_buffers.push_back((float*)malloc(buffer_size));

            if(!m_clients[i]->start_ddc1(m_ddc1_index,m_ddc1_sample_sets_per_buffer,m_ddc1_sample_sets_to_send))
            {
                while(i--)
                {
                    m_clients[i]->stop_ddc1();
                }

                free_buffers();

                wxMessageBox(wxT("Failed to start DDC1!"),APP_TITLE,wxICON_ERROR,this);
                m_bddc1->SetValue(true);
                return;
            }
        }

        trigger();
    }
    else
    {
        for(auto client:m_clients)
        {
            client->stop_ddc1();
        }

        free_buffers();

        m_ddc1_spectrum->Clear();
    }

    update_state();
}

void main_window::update_state(void)
{
 TPhase **phases;

    if(m_connected)
    {
        m_bpower->SetValue(m_power);
    }
    else
    {
        m_bpower->SetValue(false);
    }

    if(!m_power)
    {
        m_ddc1_spectrum->Clear();

        for(uint32_t i=0;i<m_clients.size();i++)
        {
            for(uint32_t j=0;j<m_clients.size();j++)
            {
                phases=m_phases[i];

                if(phases[j])
                {
                    phases[j]->Clear();
                }
            }
        }

        m_bddc1->SetValue(false);

        for(uint32_t i=0;i<m_clients.size();i++)
        {
            m_lstate_value[i]->SetLabel(wxT("N/A"));
            m_l1pps_event_counter_value[i]->SetLabel(wxT("N/A"));
            m_l1pps_adc_period_counter_value[i]->SetLabel(wxT("N/A"));
            m_l1pps_ddc1_sample_counter_value[i]->SetLabel(wxT("N/A"));
            m_l1pps_ddc1_adc_counter_value[i]->SetLabel(wxT("N/A"));
        }
    }



    m_bpower->Enable(m_connected);
    m_b1pps->Enable(m_connected && !m_power);
    m_bddc1->Enable(m_power);
    m_cbddc1_bw->Enable(m_power && !m_bddc1->GetValue());
    m_ddc1_spectrum->Enable(m_power);
    m_eddc1_frequency->Enable(m_power);
    m_lddc1_rbw->Enable(m_power);
    m_latten->Enable(m_power);
    m_cbatten->Enable(m_power);
    m_bpreamp->Enable(m_power);

    for(auto button:m_rx_buttons)
    {
        button->Enable(m_power);
    }

    for(uint32_t i=0;i<m_clients.size();i++)
    {
        m_lstate[i]->Enable(m_power);
        m_lstate_value[i]->Enable(m_power);

        m_l1pps_event_counter[i]->Enable(m_power);
        m_l1pps_event_counter_value[i]->Enable(m_power);

        m_l1pps_adc_period_counter[i]->Enable(m_power);
        m_l1pps_adc_period_counter_value[i]->Enable(m_power);

        m_l1pps_ddc1_sample_counter[i]->Enable(m_power);
        m_l1pps_ddc1_sample_counter_value[i]->Enable(m_power);

        m_l1pps_ddc1_adc_counter[i]->Enable(m_power);
        m_l1pps_ddc1_adc_counter_value[i]->Enable(m_power);

        phases=m_phases[i];

        for(uint32_t j=0;j<m_clients.size();j++)
        {
            if(phases[j])
            {
                phases[j]->Enable(m_power);
            }
        }
    }
}

void main_window::on_timer(wxTimerEvent& event)
{
    switch(event.GetId())
    {
        case ID_TIMER:
            m_config_window->update_serial_numbers();
            update_device_states();
            break;
    }
}

void main_window::free_buffers(void)
{
    while(!m_ddc1_buffers.empty())
    {
        free(m_ddc1_buffers.back());
        m_ddc1_buffers.pop_back();
    }
}

void main_window::on_ddc1_bw_chaned(wxCommandEvent &event)
{
    update_ddc1_bw();
}


void main_window::update_ddc1_bw(void)
{
 wxString s;
 int rbw;

    m_ddc1_index=m_cbddc1_bw->GetSelection();
    m_ddc1_info=ddc_infos[m_ddc1_index];

    m_ddc1_spectrum->SetRange(m_ddc1_info.bandwidth);

    m_ddc1_sample_sets_per_buffer=m_ddc1_info.sample_rate*UPDATE_INTERVAL/1000;

    if(m_ddc1_info.bandwidth<=4000000)
    {
        m_ddc1_sample_sets_per_buffer=(m_ddc1_sample_sets_per_buffer+63)&(~63);
    }
    else
    {
        m_ddc1_sample_sets_per_buffer=(m_ddc1_sample_sets_per_buffer+1023)&(~1023);
    }

    m_ddc1_sample_sets_to_send=m_ddc1_sample_sets_per_buffer>=MAX_DDC1_SAMPLE_SET_COUNT?MAX_DDC1_SAMPLE_SET_COUNT:m_ddc1_sample_sets_per_buffer;

    m_fft_size=MAX_FFT_SIZE;

    while(m_fft_size>m_ddc1_sample_sets_to_send)
    {
        m_fft_size/=2;
    }

    rbw=(int)((double)(int)m_ddc1_info.sample_rate/(int)m_fft_size+0.5);

    if(rbw>1000)
    {
        rbw+=50;
        rbw=(rbw/100)*100;
    }

    s.sprintf(wxT("RBW %s"),format_frequency(rbw).c_str());

    m_lddc1_rbw->SetLabel(s);
}

void main_window::on_ddc1_frequency_changed(wxNotifyEvent &event)
{
    m_ddc1_frequency=(int)m_eddc1_frequency->Get();

    m_ddc1_spectrum->SetCenterFrequency(m_ddc1_frequency);

    for(auto client:m_clients)
    {
        client->set_ddc1_frequency(m_ddc1_frequency);
    }

    trigger();
}

void main_window::on_atten_chaned(wxCommandEvent &event)
{
    set_atten();
}

void main_window::set_atten(void)
{
 uint32_t atten=m_cbatten->GetSelection()*3;

    for(auto client:m_clients)
    {
        client->set_atten(atten);
    }
}

void main_window::on_click_preamp(wxCommandEvent &event)
{
    set_preamp();
}

void main_window::set_preamp(void)
{
 uint8_t preamp=m_bpreamp->GetValue()?1:0;

    for(auto client:m_clients)
    {
        client->set_preamp(preamp);
    }
}

void main_window::on_click_1pps(wxCommandEvent &event)
{
    set_1pps();
}

void main_window::set_1pps(void)
{
 uint8_t enabled=m_b1pps->GetValue()?1:0;

    for(auto client:m_clients)
    {
        client->set_1pps_enabled(enabled);
    }
}


void main_window::update_rx_button(uint32_t index)
{
 TPhase **phases;

    m_rx_buttons[index]->SetLabel(m_clients[index]->get_remote_host_desc());

    for(uint32_t i=0;i<m_clients.size();i++)
    {
        phases=m_phases[i];

        for(uint32_t j=0;j<m_clients.size();j++)
        {
            if(phases[j] && index==j)
            {
                phases[j]->SetLabel(m_clients[j]->get_remote_host_desc());
            }
        }
    }
}

void main_window::on_click_rx_button(wxCommandEvent &event)
{
 uint32_t index,i;

    index=event.GetId()-ID_RX_BUTTON;

    if(m_rx_buttons[index]->GetValue())
    {
        for(i=0;i<m_clients.size();i++)
        {
            if(i!=index)
            {
                m_rx_buttons[i]->SetValue(false);
            }
        }

        m_phase_sizers[m_active_rx_index]->ShowItems(false);

        m_active_rx_index=index;

        m_phase_sizers[m_active_rx_index]->ShowItems(true);

        Layout();
    }
    else
    {
        m_rx_buttons[index]->SetValue(true);
    }
}

void main_window::get_phase(float *buffer1,float *buffer2,uint32_t count,float *min,float *max)
{
 uint32_t i;
 float i1,q1,i2,q2;
 float p;

    i1=buffer1[0];
    q1=buffer1[1];
    i2=buffer2[0];
    q2=buffer2[1];

    p=atan2f((i1*q2-q1*i2),(i1*i2+q1*q2))*180.0f/M_PI;

    *min=*max=p;

    for(i=1;i<count;i++)
    {
        i1=buffer1[i*2+0];
        q1=buffer1[i*2+1];
        i2=buffer2[i*2+0];
        q2=buffer2[i*2+1];

        p=atan2f((i1*q2-q1*i2),(i1*i2+q1*q2))*180.0f/M_PI;

        if(p<*min)
        {
            *min=p;
        }
        else if(p>*max)
        {
            *max=p;
        }
    }
}


void main_window::rx_client_callback_ddc1_samples(rx_client *client,uint64_t index,float *samples,uint32_t count,uint32_t bits_per_sample)
{
 uint32_t half,i;
 double m;
 TPhase **phases;
 float min,max;

    if(client==m_clients[m_active_rx_index])
    {
        for(i=0;i<m_fft_size;i++)
        {
            m_fft_buffer[i*2]=samples[i*2]*m_fft_coeffs[i];
            m_fft_buffer[i*2+1]=samples[i*2+1]*m_fft_coeffs[i];
        }

        FFT(m_fft_buffer,m_fft_size);

        for(i=0;i<m_fft_size;i++)
        {
            m=m_fft_buffer[i*2]*m_fft_buffer[i*2]+m_fft_buffer[i*2+1]*m_fft_buffer[i*2+1];

            if(m>0)
            {
                m_fft_buffer[i]=10*log10(m);
            }
            else
            {
                m_fft_buffer[i]=-180;
            }
        }

        half=m_fft_size/2;
        for(i=0;i<half;i++)
        {
            std::swap(m_fft_buffer[i],m_fft_buffer[i+half]);
        }

        m_ddc1_spectrum->SetData(m_fft_buffer,m_fft_size,m_ddc1_info.sample_rate,(m_ddc1_info.sample_rate-m_ddc1_info.bandwidth)/2);
    }

    m_ddc1_sample_set_index_cs.Enter();

    auto it=m_ddc1_sample_set_index.find(index);

    if(it!=m_ddc1_sample_set_index.end())
    {
        it->second++;

        if(it->second==m_clients.size())
        {
            for(uint32_t i=0;i<m_clients.size();i++)
            {
                m_clients[i]->get_buffer(index,m_ddc1_buffers[i]);
            }

            for(uint32_t i=0;i<m_clients.size();i++)
            {
                phases=m_phases[i];

                for(uint32_t j=0;j<m_clients.size();j++)
                {
                    if(i!=j)
                    {
                        get_phase(m_ddc1_buffers[i],m_ddc1_buffers[j],m_ddc1_sample_sets_to_send,&min,&max);

                        phases[j]->Add(min,max);
                    }
                }
            }

            m_ddc1_sample_set_index.erase(it);
        }
    }
    else
    {
        m_ddc1_sample_set_index[index]=1;
    }

    m_ddc1_sample_set_index_cs.Leave();
}


wxString thousands_separator(const wxString &in)
{
 wxString out=in;
 size_t size=out.size();
 ssize_t separator;

    separator=in.rfind(wxT('.'));

    if(separator>=0)
    {
        size=separator;
    }

    if(size>3)
    {
        out.insert(size-3,wxT(','));
        size++;

        if(size>7)
        {
            out.insert(size-7,wxT(','));
            size++;

            if(size>11)
            {
                out.insert(size-11,wxT(','));
            }
        }
    }

    return out;
}

void main_window::update_device_states(void)
{
 G65DDC_DEVICE_STATE device_state;
 uint32_t event_counter;
 uint32_t adc_period_counter;
 double ddc1_sample_counter;
 uint64_t ddc1_adc_counter;
 wxString s;

    if(m_power)
    {
        for(size_t i=0;i<m_clients.size();i++)
        {
            m_clients[i]->get_state(&device_state,&event_counter,&adc_period_counter,&ddc1_sample_counter,&ddc1_adc_counter);

            s=wxEmptyString;

            if(device_state.Flags & G65DDC_DEVICE_STATE_NO_1PPS)
            {
                s=wxT("NO 1PPS");
            }

            if(device_state.Flags & G65DDC_DEVICE_STATE_NOT_ARMED)
            {
                if(!s.IsEmpty())
                {
                    s+=wxT(" / ");
                }

                s+=wxT("NOT ARMED");
            }

            if(device_state.Flags & G65DDC_DEVICE_STATE_TRIGGER_NOT_SENT)
            {
                if(!s.IsEmpty())
                {
                    s+=wxT(" / ");
                }

                s+=wxT("TRIGGER NOT SENT");
            }

            if(device_state.Flags & G65DDC_DEVICE_STATE_NOT_TRIGGERED)
            {
                if(!s.IsEmpty())
                {
                    s+=wxT(" / ");
                }

                s+=wxT("NOT TRIGGERED");
            }

            if(device_state.Flags & G65DDC_DEVICE_STATE_HIGH_TEMPERATURE)
            {
                if(!s.IsEmpty())
                {
                    s+=wxT(" / ");
                }

                s+=wxT("HIGH TEMPERATURE");
            }

            if(s.IsEmpty())
            {
                s=wxT("OK");
            }

            m_lstate_value[i]->SetLabel(s);

            if(event_counter!=0xFFFFFFFF)
            {
                s.sprintf("%u",event_counter);
                m_l1pps_event_counter_value[i]->SetLabel(thousands_separator(s));

                s.sprintf("%u",adc_period_counter);
                m_l1pps_adc_period_counter_value[i]->SetLabel(thousands_separator(s));
            }
            else
            {
                m_l1pps_event_counter_value[i]->SetLabel(wxT("N/A"));
                m_l1pps_adc_period_counter_value[i]->SetLabel(wxT("N/A"));
            }

            if(ddc1_sample_counter>=0)
            {
                s.sprintf("%.3f",ddc1_sample_counter);
                m_l1pps_ddc1_sample_counter_value[i]->SetLabel(thousands_separator(s));

                s.sprintf("%I64u",ddc1_adc_counter);
                m_l1pps_ddc1_adc_counter_value[i]->SetLabel(thousands_separator(s));
            }
            else
            {
                m_l1pps_ddc1_sample_counter_value[i]->SetLabel(wxT("N/A"));
                m_l1pps_ddc1_adc_counter_value[i]->SetLabel(wxT("N/A"));
            }
        }
    }
}

void main_window::on_phase_shift_changed(wxNotifyEvent &event)
{
 uint32_t index;

    index=event.GetId()-ID_PHASE_SHIFT;

    m_clients[index]->set_ddc1_phase_shift((double)m_ephase_shift[index]->Get()*0.01);
}
