/* ***********************************************************************/
/*                                                                       */
/*                    WiNRADiO G65DDC EXCALIBUR SIGMA                    */
/*         Ioctl codes and definitions for G65DDC device driver          */
/*                     (c) Copyright 2023 RADIXON.                       */
/*                                                                       */
/* ***********************************************************************/

#ifndef __G65DDC_IOCTL_H__
#define __G65DDC_IOCTL_H__

#define INTERNAL_DRIVER_VERSION         1

#define BUS_TYPE_PCIE                   0
#define BUS_TYPE_USB2                   1
#define BUS_TYPE_USB3                   2

#define G65DDC_USB_VENDOR_ID            0x14E0
#define G65DDC_USB_PRODUCT_ID           0x1305

#define G65DDC_PCI_VENDOR_ID            0x186F
#define G65DDC_PCI_DEVICE_ID            0x1405

#define USB_LED_MODE_OFF                0
#define USB_LED_MODE_ON                 1
#define USB_LED_MODE_DIAG               2

#define USB_LED_PATTERN_OFF             0x0000
#define USB_LED_PATTERN_ON              0xFFFF
#define USB_LED_PATTERN_NO_DRIVER       0xFCCC
#define USB_LED_PATTERN_POWER_OFF       0x0033
#define USB_LED_PATTERN_POWER_ON        0xFFCC
#define USB_LED_PATTERN_APP_NOT_RUNNING 0x0333

#define CRITICAL_TEMPERATURE_0          77
#define CRITICAL_TEMPERATURE_1          72
#define CRITICAL_TEMPERATURE_2          70

#define PCI_BUFFER_SIZE_ALIGNMENT       256
#define PCI_BUFFER_SIZE_ALIGNMENT_MASK  (PCI_BUFFER_SIZE_ALIGNMENT-1)
#define PCI_BUFFER_MAX_SIZE             (16*1024*1024)

#define PCI_DMA_CHANNEL_COUNT_FROM_DEVICE 7
#define PCI_DMA_CHANNEL_COUNT_TO_DEVICE 2

#define PCI_BUFFER_SIZE_ALIGNMENT       256
#define PCI_BUFFER_SIZE_ALIGNMENT_MASK  (PCI_BUFFER_SIZE_ALIGNMENT-1)
#define PCI_BUFFER_MAX_SIZE             (16*1024*1024)

#define DMA_DIRECTION_FROM_DEVICE       0
#define DMA_DIRECTION_TO_DEVICE         1

#define PCI_SPI_CS_NONE                 0x00
#define PCI_SPI_CS_FLASH_MEM            0x01
#define PCI_SPI_CS_ATMEL                0x02

#pragma pack(push,1)

union g65ddc_state
{
    struct
    {
        uint32_t    connected:1;
        uint32_t    power:4;
        uint32_t    critical_temperature:1;
        uint32_t    suspended:1;
    } s;

    uint32_t    value;
};

struct g65ddc_ioctl_info
{
    uint16_t    driver_version;
    uint8_t     bus_type;
    uint16_t    vid_pid[4];
    uint16_t    fw[4];
};

struct g65ddc_ioctl_usb_ep
{
    uint8_t     address;
    uint16_t    max_packet_size;
    uint8_t     interval;
    uint8_t     type;
};

struct g65ddc_ioctl_usb_ctrl_vendor_msg
{
    uint8_t     request;
    uint16_t    value;
    uint16_t    index;
    void        *data;
    uint16_t    size;
};

struct g65ddc_ioctl_usb_bulk_msg
{
    uint32_t    ep_index;
    void        *data;
    uint32_t    size;
};

struct g65ddc_ioctl_usb_add_ep_buffer
{
    uint32_t    ep_index;
    void        *buffer;
    uint32_t    size;
};

struct g65ddc_ioctl_usb_ep_requeue_buffer
{
    uint32_t    ep_index;
    uint32_t    transfer_size;
};

union g65ddc_ioctl_usb_ep_complete_buffer
{
    uint32_t    ep_index;
    void        *buffer;
};

struct g65ddc_ioctl_pci_dma_start_params
{
    uint32_t    buffer_size;
    uint16_t    direction;
    uint16_t    channel_index;
};

struct g65ddc_ioctl_pci_dma_queue_buffer
{
    uint16_t    direction;
    uint16_t    channel_index;
    void        *buffer;
    uint32_t    size;
};

struct g65ddc_ioctl_pci_dma_channel
{
    uint16_t    direction;
    uint16_t    channel_index;
};

union g65ddc_ioctl_pci_dma_complete_buffer
{
    struct g65ddc_ioctl_pci_dma_channel channel;
    void    *buffer;
};


#pragma pack(pop)

#define G65DDC_IOCTL_GET_INFO                       _IOR(0x8C,0x00,struct g65ddc_ioctl_info)
#define G65DDC_IOCTL_GET_STATE                      _IOWR(0x8C,0x01,union g65ddc_state)
#define G65DDC_IOCTL_SET_POWER                      _IO(0x8C,0x02)

#define G65DDC_IOCTL_USB_GET_EP_COUNT               _IO(0x8C,0x10)
#define G65DDC_IOCTL_USB_GET_EP_INFO                _IOR(0x8C,0x11,struct g65ddc_ioctl_usb_ep)

#define G65DDC_IOCTL_USB_CTRL_VENDOR_MSG_RCV        _IOW(0x8C,0x17,struct g65ddc_ioctl_usb_ctrl_vendor_msg)
#define G65DDC_IOCTL_USB_CTRL_VENDOR_MSG_SND        _IOW(0x8C,0x18,struct g65ddc_ioctl_usb_ctrl_vendor_msg)
#define G65DDC_IOCTL_USB_BULK_MSG_RCV               _IOW(0x8C,0x19,struct g65ddc_ioctl_usb_bulk_msg)
#define G65DDC_IOCTL_USB_BULK_MSG_SND               _IOW(0x8C,0x1A,struct g65ddc_ioctl_usb_bulk_msg)
#define G65DDC_IOCTL_USB_SET_LED_MODE               _IO(0x8C,0x1B)

#define G65DDC_IOCTL_USB_EP_CLEAR_HALT              _IO(0x8C,0x30)
#define G65DDC_IOCTL_USB_EP_RESET                   _IO(0x8C,0x31)

#define G65DDC_IOCTL_USB_EP_ADD_BUFFER              _IOW(0x8C,0x33,struct g65ddc_ioctl_usb_add_ep_buffer)
#define G65DDC_IOCTL_USB_EP_CANCEL_BUFFERS          _IO(0x8C,0x34)
#define G65DDC_IOCTL_USB_EP_RELEASE_BUFFERS         _IO(0x8C,0x35)
#define G65DDC_IOCTL_USB_EP_REQUEUE_BUFFER          _IOW(0x8C,0x36,struct g65ddc_ioctl_usb_ep_requeue_buffer)
#define G65DDC_IOCTL_USB_EP_GET_COMPLETE_BUFFER     _IOWR(0x8C,0x37,union g65ddc_ioctl_usb_ep_complete_buffer)

#define G65DDC_IOCTL_PCI_DMA_START                  _IOW(0x8C,0x50,struct g65ddc_ioctl_pci_dma_start_params)
#define G65DDC_IOCTL_PCI_DMA_STOP                   _IOW(0x8C,0x51,struct g65ddc_ioctl_pci_dma_channel)
#define G65DDC_IOCTL_PCI_DMA_QUEUE_BUFFER           _IOW(0x8C,0x52,struct g65ddc_ioctl_pci_dma_queue_buffer)
#define G65DDC_IOCTL_PCI_DMA_GET_COMPLETE_BUFFER    _IOWR(0x8C,0x53,union g65ddc_ioctl_pci_dma_complete_buffer)



#endif // __G65DDC_IOCTL_H__

