#!/bin/bash

VERSION="1.05"
DEVICE_NAME="G65DDC"
DEPMOD="/sbin/depmod"
LSMOD="/sbin/lsmod"
MODULE_NAME="g65ddc"
RMMOD="/sbin/rmmod"
KERNEL_RELEASE=`uname -r`
API="libg65ddcapi.so"
API_V="$API.$VERSION"
PLATFORM=`uname -m`
LD=/lib/ld-linux.so.2
LD_x86_64=/lib64/ld-linux-x86-64.so.2
PLATFORMS="i386 i486 i586 i686 x86_64"
LDS="${LD} ${LD} ${LD} ${LD} ${LD_x86_64}"

  echo
  echo "**********************************************************************"
  echo "*                  WiNRADiO G65DDC EXCALIBUR SIGMA                   *"
  echo "*                     Driver and API uninstaller                     *"
  echo "**********************************************************************"
  echo

  if [[ $UID -ne 0 ]] ; then
    echo "root privileges are required."
    echo 
    echo "Press ENTER to exit."
    read
    exit 0 
  fi

  echo "If the WiNRADiO $DEVICE_NAME driver and API are uninstalled,"
  echo "the applications that use them will not work."
  echo -n "Are you sure you want to uninstall $DEVICE_NAME driver and API? [n] "
  read ANSWER

  case $ANSWER in 
    "y");;
    "Y");;
    *)
      echo
      echo "WiNRADiO $DEVICE_NAME uninstaller aborted."
      exit 0;;
  esac  

  echo

  MODULE_LOADED=`$LSMOD | grep $MODULE_NAME | wc -l`  

  if [[ $MODULE_LOADED -ne 0 ]]; then
    echo -n "Unloading $DEVICE_NAME kernel module..."

    $RMMOD $MODULE_NAME >/dev/null 2>&1

    if [[ $? -ne 0 ]] ; then
      echo -e "\r* ERROR: Unable to unload $DEVICE_NAME kernel module. Please, close all "
      echo "* the applications that use $DEVICE_NAME devices and try the uninstaller again."
      echo 
      echo "Press ENTER to exit."
      read
      exit 0
    fi

    echo -e "\r$DEVICE_NAME kernel module has been unloaded."
  fi
  
  echo -n "Uninstalling $DEVICE_NAME kernel module..."
  
  cd driver
  make uninstall >/dev/null 2>&1
  cd ..
  
  echo -e "\r$DEVICE_NAME kernel module has been uninstalled."

  echo -e "Uninstalling $DEVICE_NAME API..."

  for p in $PLATFORMS ; do
     f="lib/$p/$API_V"
     if [[ -f "$f" ]] ; then
        lib_dir=
        for l in $LDS ; do
            if test -x $l ; then
                $l --verify "$f" >/dev/null 2>&1
                r=$?
                case $r in
                    0|2)
                        lib_dir=$($l --list "$f" 2>/dev/null | grep /libstdc++.so | sed 's/.*\s\//\//g' | sed 's/\/libstdc++.*//g')

                        if [[ "$lib_dir" != "" ]] ; then
                            if test -f "$lib_dir/$API" ; then
                                 if test -L "$lib_dir/$API" ; then
                                     tmp=$(readlink -f "$lib_dir/$API")
                                     if test -f "$tmp" ; then
                                         rm -f "$tmp" >/dev/null 2>&1
                                     fi
                                 fi
                                 rm -f "$lib_dir/$API" >/dev/null 2>&1
                            fi
                        fi
                        break;
                    ;;
                    *)
                    ;;
                esac
            fi
        done
     fi
  done

  echo
  echo "Done."
  echo
  echo "Press ENTER to exit."
  read